package com.ruoyi.ss.room.service.impl;

import cn.hutool.core.lang.Opt;
import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.feeRule.domain.FeeRule;
import com.ruoyi.ss.feeRule.domain.FeeRuleVO;
import com.ruoyi.ss.feeRule.service.IFeeRuleService;
import com.ruoyi.ss.order.domain.vo.ReservedTimePeriod;
import com.ruoyi.ss.order.service.IOrderService;
import com.ruoyi.ss.room.domain.RoomVO;
import com.ruoyi.ss.room.service.IRoomAssembler;
import com.ruoyi.ss.store.domain.StoreVO;
import com.ruoyi.ss.store.service.IStoreAssembler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author wjh
 * 2024/4/29
 */
@Service
public class RoomAssemblerImpl implements IRoomAssembler {

    @Autowired
    private IFeeRuleService feeRuleService;

    @Autowired
    private IOrderService orderService;

    /**
     * 拼接设备数量
     *
     * @param list 店铺列表
     */
    @Override
    public void assembleRuleInfo(List<RoomVO> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        list.forEach(room -> {
            // 每个小时价格和最低套餐
            assembleRule(room);

            // 已售数
            int soldNum = orderService.selectSoldNum(room.getRoomId());
            room.setSoldNum(soldNum);

            //已预定时间段(包含)
            assembleReservedTimePeriods(room);
        });
    }

    private void assembleReservedTimePeriods(RoomVO room) {
        //以当前时间开始的24小时内的已经预定成功的订单时间，预定成功的判断是  order.status == 2 的订单 ,只返回 order.reserveStartTime（Date）和reserveEndTime（Date）,放在list中，list的结构：
        // [{"startTime":"2024-04-29 14:00:00","endTime":"2024-04-29 15:00:00"}, {"startTime":"2024-04-29 16:00:00","endTime":"2024-04-29 17:00:00"}]
        // 获取当前时间
        Date startTime = new Date();

        // 获取24小时后的时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startTime);
        calendar.add(Calendar.HOUR, 24);
        Date endTime = calendar.getTime();

        // 获取已预定时间段：预定成功的订单，时间在当前时间的24小时内
        List<ReservedTimePeriod> reservedTimes = orderService.selectReservedTimes(room.getRoomId(), startTime, endTime);
        room.setReservedTimePeriods(reservedTimes);
    }

    private void assembleRule(RoomVO room) {
        List<FeeRuleVO> feeRuleVOS = feeRuleService.selectEFeeRuleListByRoomId(room.getRoomId());
        // 检查 feeRuleVOS 是否为空
        if (feeRuleVOS == null || feeRuleVOS.isEmpty()) {
            return; // 如果为空则跳过处理
        }

        // 处理逻辑：优先获取 mode 为 "1" 的 FeeRuleVO
        FeeRuleVO modeOneFeeRule = feeRuleVOS.stream()
                .filter(feeRule -> "1".equals(feeRule.getMode())) // 筛选 mode 为 "1"
                .findFirst() // 获取第一个符合条件的 FeeRuleVO
                .orElse(null); // 如果没有找到，返回 null

        if (modeOneFeeRule != null) {
            // 如果找到了 mode 为 "1" 的记录，直接设置 hour
            room.setHour(modeOneFeeRule.getPrice());
        } else {
            // 如果没有 mode 为 "1" 的记录，计算 price / hours
            feeRuleVOS.stream()
                    .filter(feeRule -> feeRule.getHours() != null && feeRule.getHours() > 0) // 确保 hours 有效
                    .map(feeRule -> feeRule.getPrice().divide(BigDecimal.valueOf(feeRule.getHours()), 2, RoundingMode.HALF_UP)) // 计算 price / hours
                    .findFirst() // 获取第一个计算结果
                    .ifPresent(room::setHour); // 设置 hour
        }

        // 套餐价格，返回： 4小时套餐：￥149.9/场
        // 获取 mode 为 "2" 且 hours 最小的 FeeRuleVO
        feeRuleVOS.stream()
                .filter(feeRule -> "2".equals(feeRule.getMode()) && feeRule.getPrice() != null) // 筛选 mode 为 "2" 且 price 不为 null
                .min(Comparator.comparing(FeeRuleVO::getPrice)) // 按照 price 升序排序并获取价格最低的 FeeRuleVO
                .ifPresent(room::setFeeRuleVO);
    }
}
